#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <pwd.h>
#include <unistd.h>
#include <rsbac/types.h>
#include <rsbac/aci_data_structures.h>
#include <rsbac/getname.h>
#include <rsbac/acl_getname.h>
#include <rsbac/syscalls.h>
#include <rsbac/error.h>
#include <rsbac/helpers.h>
#include "locale.h"

int verbose=0;
int recurse=0;
int printall=0;
int effective=1;
int scripting=0;
int exit_res=0;
union rsbac_attribute_value_t value;
enum rsbac_target_t target;
enum rsbac_attribute_t attr;
char * progname;
enum rsbac_acl_subject_type_t subj_type=ACLS_USER;
rsbac_acl_subject_id_t subj_id;

int process(char * name)
  {
    int res = 0;
    char tmp1[120];
    enum rsbac_target_t type=T_NONE;
    char type_n[5] = "";
    struct stat buf;
    struct rsbac_acl_syscall_arg_t   arg;
    struct rsbac_acl_syscall_n_arg_t arg_n;
    rsbac_acl_rights_vector_t rights_vector = 0;

    if(!strcmp(name,":DEFAULT:"))
      {
        if(target == T_FD)
          type = T_FILE;
        else
          type = target;
        switch(type)
          {
            case T_FILE:
              strcpy(type_n,"FILE");
              break;
            case T_DIR:
              strcpy(type_n,"DIR");
              break;
            case T_FIFO:
              strcpy(type_n,"FIFO");
              break;
            case T_DEV:
              strcpy(type_n,"DEV");
              break;
            case T_IPC:
              strcpy(type_n,"IPC");
              arg.tid.ipc.type = I_none;
              break;
            case T_SCD:
              strcpy(type_n,"SCD");
              arg.tid.scd = AST_none;
              break;
            case T_USER:
              strcpy(type_n,"USER");
              arg.tid.user = RSBAC_NO_USER;
              break;
            case T_PROCESS:
              strcpy(type_n,"PROCESS");
              arg.tid.process = 0;
              break;
            default:
              fprintf(stderr, gettext("Invalid target %u!\n"),
                      target);
              return(1);
          }
        arg_n.name = NULL;
      }
    else
      {
        switch(target)
          {
            case T_FILE:
            case T_DIR:
            case T_FIFO:
            case T_FD:
            case T_DEV:
              if (lstat(name,&buf))
                {
                  fprintf(stderr, gettext("stat for file %s returned error: %s\n"),
                          name,
                          strerror(errno));
                  return(1);
                }
            default:
              break;
          }
        arg_n.name = name;
        switch(target)
          {
            case T_FILE:
              if(   (S_ISREG(buf.st_mode))
                 || (S_ISBLK(buf.st_mode))
                 || (S_ISCHR(buf.st_mode)) )
                {
                  type = T_FILE;
                  strcpy(type_n,"FILE");
                }
              else
                {
                  if(verbose)
                    printf(gettext("%s is no file, skipped\n"),
                           name);
                  goto do_recurse;
                }
              break;
            case T_DIR:
              if(S_ISDIR(buf.st_mode))
                {
                  type = T_DIR;
                  strcpy(type_n,"DIR");
                }
              else
                {
                  if(verbose)
                    printf(gettext("%s is no dir, skipped\n"),
                           name);
                  goto do_recurse;
                }
              break;
            case T_FIFO:
              if(S_ISFIFO(buf.st_mode))
                {
                  type = T_FIFO;
                  strcpy(type_n,"FIFO");
                }
              else
                {
                  if(verbose)
                    printf(gettext("%s is no fifo, skipped\n"),
                           name);
                  goto do_recurse;
                }
              break;
            case T_FD:
              if(S_ISDIR(buf.st_mode))
                {
                  type = T_DIR;
                  strcpy(type_n,"DIR");
                }
              else
              if(   (S_ISREG(buf.st_mode))
                 || (S_ISBLK(buf.st_mode))
                 || (S_ISCHR(buf.st_mode)) )
                {
                  type = T_FILE;
                  strcpy(type_n,"FILE");
                }
              else
              if(S_ISFIFO(buf.st_mode))
                {
                  type = T_FIFO;
                  strcpy(type_n,"FIFO");
                }
              else
                {
                  if(verbose)
                    printf(gettext("%s is neither file nor dir nor fifo, skipped\n"),
                           name);
                  goto do_recurse;
                }
              break;
            case T_DEV:
              if(   (S_ISBLK(buf.st_mode))
                 || (S_ISCHR(buf.st_mode)) )
                {
                  type = T_DEV;
                  strcpy(type_n,"DEV");
                }
              else
                {
                  if(verbose)
                    printf(gettext("%s is no file, skipped\n"),
                           name);
                  goto do_recurse;
                }
              break;
            case T_SCD:
              arg.tid.scd = get_acl_scd_type_nr(name);
              if((arg.tid.scd == ST_none) || (arg.tid.scd == AST_none))
                {
                  fprintf(stderr, gettext("%s is no valid SCD name, skipped\n"),
                           name);
                  return(1);
                }
              break;

            default:
              fprintf(stderr, gettext("Invalid target %u for %s, skipped!\n"),
                      target, name);
              return(1);
          }
      }
    if(verbose)
    printf(gettext("Processing %s '%s'\n"),
           type_n,
           name);
    switch(target)
      {
        case T_FILE:
        case T_DIR:
        case T_FIFO:
        case T_FD:
        case T_DEV:
          arg_n.target = type;
          arg_n.subj_type = subj_type;
          arg_n.subj_id = subj_id;
          res = rsbac_acl_get_rights_n(&arg_n, &rights_vector, effective);
          break;

        default:
          arg.target = target;
          arg.subj_type = subj_type;
          arg.subj_id = subj_id;
          res = rsbac_acl_get_rights(&arg, &rights_vector, effective);
      }
    if(res)
      {
        get_error_name(tmp1,res);
        fprintf(stderr, gettext("%s: error: %s\n"), name, tmp1);
        exit_res=1;
      }
    else
      {
        if(scripting)
          {
            if(printall)
              {
                int i;

                for (i=0; i<R_NONE; i++)
                  if(rights_vector & ((rsbac_acl_rights_vector_t) 1 << i))
                    printf("%s\n", get_request_name(tmp1,i));
                for (i=RSBAC_ACL_SPECIAL_RIGHT_BASE; i<ACLR_NONE; i++)
                  if(rights_vector & ((rsbac_acl_rights_vector_t) 1 << i))
                    printf("%s\n", get_acl_special_right_name(tmp1,i));
              }
            else
              {
                printf("%s\n",
                       u64tostracl(tmp1, rights_vector));
              }
          }
        else
          {
            printf("%-19s: %s\n",
                   name, u64tostracl(tmp1, rights_vector));
            if(printall)
              {
                int i;

                for (i=0; i<R_NONE; i++)
                  if(rights_vector & ((rsbac_acl_rights_vector_t) 1 << i))
                    printf("  %s\n", get_request_name(tmp1,i));
                for (i=RSBAC_ACL_SPECIAL_RIGHT_BASE; i<ACLR_NONE; i++)
                  if(rights_vector & ((rsbac_acl_rights_vector_t) 1 << i))
                    printf("  %s\n", get_acl_special_right_name(tmp1,i));
              }
          }
      }

do_recurse:
    if (   arg_n.name
        && (type == T_DIR)
        && recurse)
      {
        DIR * dir_stream_p;
        struct dirent * dirent_p;
        char name2[512];

        if(S_ISLNK(buf.st_mode))
          return(0);
        if(!(dir_stream_p = opendir(name)))
          {
            fprintf(stderr, gettext("opendir for dir %s returned error: %s\n"),
                   name,
                   strerror(errno));
            return(-2);
          }
        while((dirent_p = readdir(dir_stream_p)))
          {
            if(   (strcmp(".",dirent_p->d_name))
               && (strcmp("..",dirent_p->d_name)) )
              {
                strcpy(name2,name);
                strcat(name2,"/");
                strcat(name2,dirent_p->d_name);
                process(name2);
              }
          }
        closedir(dir_stream_p);
      }
    return(0);
  }

int main(int argc, char ** argv)
{
  int i;
  struct passwd * user_info_p;

  setlocale (LC_ALL, "");
  bindtextdomain (PACKAGE, LOCALEDIR);
  textdomain (PACKAGE);

  progname = argv[0];
  subj_id=getuid();
  while((argc > 1) && (argv[1][0] == '-'))
    {
      char * pos = argv[1];
      pos++;
      while(*pos)
        {
          switch(*pos)
            {
              case 'v':
                verbose=1;
                break;
              case 'r':
                recurse=1;
                break;
              case 'p':
                printall=1;
                break;
              case 's':
                scripting=1;
                break;
              case 'd':
                effective=0;
                break;
              case 'n':
                {
                  char tmp[80];

                  for(i=0; i<ST_none; i++)
                    printf("%s\n", get_scd_type_name(tmp, i));
                  for(i=AST_min; i<AST_none; i++)
                    printf("%s\n", get_acl_scd_type_name(tmp, i));
                  exit(0);
                }
              case 'N':
                {
                  char tmp[80];
                  rsbac_acl_rights_vector_t rights_vector = RSBAC_ALL_REQUEST_VECTOR;

                  if(argc > 2)
                    {
                      target = get_target_nr(argv[2]);
                      switch(target)
                        {
                          case T_FD:
                          case T_FILE:
                          case T_DIR:
                          case T_FIFO:
                            rights_vector = RSBAC_FD_REQUEST_VECTOR;
                            break;
                          case T_DEV:
                            rights_vector = RSBAC_DEV_REQUEST_VECTOR;
                            break;
                          case T_IPC:
                            rights_vector = RSBAC_IPC_REQUEST_VECTOR;
                            break;
                          case T_SCD:
                            rights_vector = RSBAC_SCD_REQUEST_VECTOR;
                            if(argc > 3)
                              {
                                enum rsbac_acl_scd_type_t type;
                                
                                type = get_acl_scd_type_nr(argv[3]);
                                if(type == ST_other)
                                  rights_vector = RSBAC_NONE_REQUEST_VECTOR;
                              }
                            break;
                          case T_USER:
                            rights_vector = RSBAC_ACL_USER_RIGHTS_VECTOR;
                            break;
                          case T_PROCESS:
                            rights_vector = RSBAC_PROCESS_REQUEST_VECTOR;
                            break;
                          default:
                            fprintf(stderr, gettext("%s: invalid target type %s for switch N\n"),
                                    progname, argv[2]);
                            exit(1);
                        }
                    }
                  for(i=0; i<R_NONE; i++)
                    if(rights_vector & RSBAC_ACL_RIGHTS_VECTOR(i))
                      printf("%s\n", get_request_name(tmp, i));
                  for(i=RSBAC_ACL_SPECIAL_RIGHT_BASE; i<ACLR_NONE; i++)
                    printf("%s\n", get_acl_special_right_name(tmp, i));
                  exit(0);
                }
              case 'u':
                if(argc < 2)
                  {
                    fprintf(stderr, gettext("%s: no user for switch u\n"), progname);
                    exit(1);
                  }
                if(   (!(user_info_p = getpwnam(argv[2])))
                   && (   (!(user_info_p = getpwuid(strtol(argv[2],0,10))))
                   || (!(user_info_p->pw_uid) && strcmp("0",argv[2])) )
                  )
                  {
                    fprintf(stderr, gettext("Invalid user %s!\n"), argv[2]);
                    exit(1);
                  }
                subj_type = ACLS_USER;
                subj_id = user_info_p->pw_uid;
                if(!scripting)
                  printf(gettext("%s: User %s\n"), progname, user_info_p->pw_name);
                argv++;
                argc--;
                break;
              case 'g':
                if(argc < 2)
                  {
                    fprintf(stderr, gettext("%s: no group for switch g\n"), progname);
                    exit(1);
                  }
                subj_type = ACLS_GROUP;
                subj_id = strtol(argv[2],0,10);
                if(!scripting)
                  printf(gettext("%s: Group %u\n"), progname, subj_id);
                argv++;
                argc--;
                break;
              case 'l':
                if(argc < 2)
                  {
                    fprintf(stderr, gettext("%s: no role for switch l\n"), progname);
                    exit(1);
                  }
                subj_type = ACLS_ROLE;
                subj_id = strtol(argv[2],0,10);
                if(!scripting)
                  printf(gettext("%s: Role %u\n"), progname, subj_id);
                argv++;
                argc--;
                break;
              case '-':
                if(!strcmp(pos,"-USER"))
                  {
                    if(argc < 2)
                      {
                        fprintf(stderr, gettext("%s: no user for switch -USER\n"), progname);
                        exit(1);
                      }
                    if(   (!(user_info_p = getpwnam(argv[2])))
                       && (   (!(user_info_p = getpwuid(strtol(argv[2],0,10))))
                       || (!(user_info_p->pw_uid) && strcmp("0",argv[2])) )
                      )
                      {
                        fprintf(stderr, gettext("Invalid user %s!\n"), argv[2]);
                        exit(1);
                      }
                    subj_type = ACLS_USER;
                    subj_id = user_info_p->pw_uid;
                    if(!scripting)
                      printf("%s: User %s\n", progname, user_info_p->pw_name);
                    argv++;
                    argc--;
                    pos+=4;
                    break;
                  }
                else
                if(!strcmp(pos,"-GROUP"))
                  {
                    if(argc < 2)
                      {
                        fprintf(stderr, gettext("%s: no group for switch -GROUP\n"), progname);
                        exit(1);
                      }
                    subj_type = ACLS_GROUP;
                    subj_id = strtol(argv[2],0,10);
                    if(!scripting)
                      printf(gettext("%s: Group %u\n"), progname, subj_id);
                    argv++;
                    argc--;
                    pos+=5;
                    break;
                  }
                else
                if(!strcmp(pos,"-ROLE"))
                  {
                    if(argc < 2)
                      {
                        fprintf(stderr, gettext("%s: no role for switch -ROLE\n"), progname);
                        exit(1);
                      }
                    subj_type = ACLS_ROLE;
                    subj_id = strtol(argv[2],0,10);
                    if(!scripting)
                      printf(gettext("%s: Role %u\n"), progname, subj_id);
                    argv++;
                    argc--;
                    pos+=4;
                    break;
                  }
                else
                  {
                    fprintf(stderr, gettext("%s: unknown parameter %s\n"), progname, pos);
                  }
                break;

              default:
                fprintf(stderr, gettext("%s: unknown parameter %c\n"), progname, *pos);
            }
          pos++;
        }
      argv++;
      argc--;
    }
  if (argc > 1)
    {
      target = get_target_nr(argv[1]);
      if(target == T_NONE)
        {
          if(verbose)
            printf(gettext("%s: %i targets\n\n"), progname, argc - 1);
          fprintf(stderr, gettext("%s: No target type given, assuming FD\n"), progname);
          target = T_FD;
          if(argc < 1)
            process(".");
          else
            for (i=1;i < (argc);i++)
              {
                process(argv[i]);
              }
        }
      else
        {
          if(argc > 2)
            {
              if(verbose)
                printf(gettext("%s: %i targets\n\n"), progname, argc - 2);
              for (i=1;i < (argc-1);i++)
                {
                  process(argv[i+1]);
                }
            }
          else
            process(".");
        }
    }
  else
    {
      printf(gettext("%s (RSBAC %s)\n***\n"), progname, VERSION);
      printf(gettext("Use: %s [switches] target-type file/dirname(s)\n"), progname);  
      printf(gettext(" -v = verbose, -r = recurse into subdirs,\n"));
      printf(gettext(" -p = print right names, -d = give direct, not effective rights\n"));
      printf(gettext(" -n = list valid SCD names, -s = scripting mode\n"));
      printf(gettext(" -N = list valid right names [for target-type]\n"));
      printf(gettext(" -u user = print rights for given user, not caller\n"));
      printf(gettext(" -g group = print rights for given group, not caller\n"));
      printf(gettext(" -l role = print rights for given role, not caller\n"));
      printf(gettext("  target-type = FILE, DIR, FIFO, DEV, IPC, SCD, USER, PROCESS or FD\n"));
      printf(gettext("  (FD: let %s decide between FILE, DIR and FIFO, no DEV),\n"), progname);
      printf(gettext("  (IPC, USER, PROCESS: only :DEFAULT:\n"));
      printf(gettext("- Use name :DEFAULT: for default ACL\n"));
    }
  return (exit_res);
}

