#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <rsbac/types.h>
#include <rsbac/aci_data_structures.h>
#include <rsbac/getname.h>
#include <rsbac/syscalls.h>
#include <rsbac/error.h>
#include <rsbac/helpers.h>
#include "locale.h"

 union rsbac_attribute_value_t value;
 enum rsbac_target_t target;
 enum rsbac_attribute_t attr;
 char * progname;
int verbose=0;
int recurse=0;
int inherit=0;

int process(char * name)
  {
    int res = 0;
    char tmp1[120];
    char tmp2[256];
    enum rsbac_target_t type = T_NONE;
    char type_n[5] = "";
    struct stat buf;

    if (lstat(name,&buf))
      {
        fprintf(stderr, gettext("stat for file %s returned error: %s\n"),
                name,
                strerror(errno));
        return(1);
      }
    switch(target)
      {
        case T_FILE:
          if(   (S_ISREG(buf.st_mode))
             || (S_ISBLK(buf.st_mode))
             || (S_ISCHR(buf.st_mode)) )
            {
              type = T_FILE;
              strcpy(type_n,"FILE");
            }
          else
            {
              if(verbose)
                printf(gettext("%s is no file, skipped\n"),
                       name);
              goto do_recurse;
            }
          break;
        case T_DIR:
          if(S_ISDIR(buf.st_mode))
            {
              type = T_DIR;
              strcpy(type_n,"DIR");
            }
          else
            {
              if(verbose)
                printf(gettext("%s is no dir, skipped\n"),
                       name);
              goto do_recurse;
            }
          break;
        case T_FIFO:
          if(S_ISFIFO(buf.st_mode))
            {
              type = T_FIFO;
              strcpy(type_n,"FIFO");
            }
          else
            {
              if(verbose)
                printf(gettext("%s is no fifo, skipped\n"),
                       name);
              goto do_recurse;
            }
          break;
        case T_FD:
          if(S_ISDIR(buf.st_mode))
            {
              type = T_DIR;
              strcpy(type_n,"DIR");
            }
          else
          if(   (S_ISREG(buf.st_mode))
             || (S_ISBLK(buf.st_mode))
             || (S_ISCHR(buf.st_mode)) )
            {
              type = T_FILE;
              strcpy(type_n,"FILE");
            }
          else
          if(S_ISFIFO(buf.st_mode))
            {
              type = T_FIFO;
              strcpy(type_n,"FIFO");
            }
          else
            {
              if(verbose)
                printf(gettext("%s is neither file nor dir nor fifo, skipped\n"),
                       name);
              goto do_recurse;
            }
          break;
        case T_DEV:
          if(   (S_ISBLK(buf.st_mode))
             || (S_ISCHR(buf.st_mode)) )
            {
              type = T_DEV;
              strcpy(type_n,"DEV");
            }
          else
            {
              if(verbose)
                printf(gettext("%s is no file, skipped\n"),
                       name);
              goto do_recurse;
            }
          break;
        default:
          fprintf(stderr, gettext("Invalid target type %u for %s, skipped!\n"),
                  target, name);
          return(1);
      }
    if(verbose)
      printf(gettext("Processing %s '%s', attribute %s\n"),
             type_n,
             name,
             get_attribute_name(tmp2,attr));
    value.dummy = -1;
    res = rsbac_get_attr_n(type, name, attr, &value, inherit);
    if(res)
      {
        get_error_name(tmp1,res);
        fprintf(stderr, gettext("error: %s\n"), tmp1);
      }
    else
      switch (attr)
        {
          case A_log_array_low:
          case A_log_array_high:
          case A_log_program_based:
            printf(gettext("%s: Returned value: %s\n"),
                   name, u64tostrlog(tmp1,value.log_array_low));
            break;
          case A_mac_categories:
            printf(gettext("%s: Returned value: %s\n"),
                   name, u64tostrmac(tmp1,value.mac_categories));
            break;
          case A_mac_trusted_for_user:
            printf(gettext("%s: Returned value: %u\n"),
                   name, value.mac_trusted_for_user);
            break;
          case A_ff_flags:
          case A_rc_type:
          case A_rc_type_fd:
          case A_rc_force_role:
          case A_rc_role:
          case A_rc_def_role:
          case A_security_level:
            printf(gettext("%s: Returned value: %u\n"),
                   name, value.u_char_dummy);
            break;
          default:
            printf(gettext("%s: Returned value: %i\n"),
                   name, value.dummy);
        }

do_recurse:
    if(   S_ISDIR(buf.st_mode)
       && recurse)
      {
        DIR * dir_stream_p;
        struct dirent * dirent_p;
        char name2[512];

        if(S_ISLNK(buf.st_mode))
          return(0);
        if(!(dir_stream_p = opendir(name)))
          {
            fprintf(stderr, gettext("opendir for dir %s returned error: %s\n"),
                   name,
                   strerror(errno));
            return(-2);
          }
        while((dirent_p = readdir(dir_stream_p)))
          {
            if(   (strcmp(".",dirent_p->d_name))
               && (strcmp("..",dirent_p->d_name)) )
              {
                strcpy(name2,name);
                strcat(name2,"/");
                strcat(name2,dirent_p->d_name);
                process(name2);
              }
          }
        closedir(dir_stream_p);
      }
    return(0);
  }


int main(int argc, char ** argv)
{
  enum rsbac_attribute_t attr_list[RSBAC_FD_NR_ATTRIBUTES] = RSBAC_FD_ATTR_LIST;
  enum rsbac_attribute_t attr_list_dev[RSBAC_DEV_NR_ATTRIBUTES] = RSBAC_DEV_ATTR_LIST;
  int res = 0;
  char tmp1[80],tmp2[120];
  int i,j;

  setlocale (LC_ALL, "");
  bindtextdomain (PACKAGE, LOCALEDIR);
  textdomain (PACKAGE);
  
  progname = argv[0];
  
  while((argc > 1) && (argv[1][0] == '-'))
    {
      char * pos = argv[1];
      pos++;
      while(*pos)
        {
          switch(*pos)
            {
              case 'v':
                verbose++;
                break;
              case 'r':
                recurse=1;
                break;
              case 'e':
                inherit=1;
                break;
              case 'n':
                {
                  char tmp[80];
                  int i;

                  for(i=0; i<R_NONE; i++)
                    printf("%s\n", get_request_name(tmp, i));
                  exit(0);
                }
              default:
                fprintf(stderr, gettext("%s: unknown parameter %c\n"), progname, *pos);
            }
          pos++;
        }
      argv++;
      argc--;
    }

  if (argc > 3)
    {
      if(verbose)
        printf(gettext("%s: %i targets\n\n"), progname, argc - 3);
      target = get_target_nr(argv[1]);
      attr = get_attribute_nr(argv[2]);
      for (i=3;i < (argc);i++)
        {
          process(argv[i]);
        }
    }
  else
    {
      printf(gettext("%s (RSBAC %s)\n***\n"), argv[0], VERSION);
      printf(gettext("Use: %s [-v] [-e] target-type attribute file/dirname(s)\n"), progname);
      printf(gettext("- -v = verbose, -e = show effective (maybe inherited) value, not real\n"));
      printf(gettext("- -r = recurse into subdirs, -n = list all requests\n"));
      printf(gettext("- target-type = FILE, DIR, FIFO, DEV or FD\n"));
      printf(gettext("  (FD: let %s decide between FILE, DIR and FIFO, no DEV),\n"), progname);
      printf(gettext("- attribute (string) and returned value = see following lists:\n"));
      printf(gettext("- FILE, DIR and FIFO:\n"));
      for (j=0;j<RSBAC_FD_NR_ATTRIBUTES;j++)
        {
          get_attribute_name(tmp1,attr_list[j]);
          get_attribute_param(tmp2,attr_list[j]);
          printf("%s \t%s\n",tmp1,tmp2);
        }
      printf("- DEV:\n");
      for (j=0;j<RSBAC_DEV_NR_ATTRIBUTES;j++)
        {
          get_attribute_name(tmp1,attr_list_dev[j]);
          get_attribute_param(tmp2,attr_list_dev[j]);
          printf("%s \t%s\n",tmp1,tmp2);
        }
    }
  return (res);
}
