/*RSBAC global network
 *
 *(c) Stanislav Ievlev <inger@linux.ru.net>
 */

/*r-jobd-clist.c - jobs server.Some work with jobs*/

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <netdb.h>
#include <syslog.h>
#include <signal.h>
#include <string.h>

#include "r-job.h"
#include "r-job-d.h"
#include "auth.h"

/*external*/
extern char *clientlist_filename;
extern char *job_filename;
extern struct sockaddr_in clnt_addr;

/*append a new job to job files*/
void append_job(char *job_line)
{
    FILE *job_file=NULL;
    FILE *cl_file=NULL;
    char tmpstr[MAX_LINE];
    char tmpstr1[MAX_LINE];
    char filename[MAX_LINE];
    
    cl_file=fopen(clientlist_filename,"r");
    
    if (cl_file==NULL)
    {
	syslog(LOG_INFO, "rsbac-net: Cannot open client-list file.");
	exit(1);
    }
    /*processing for all clients*/
    while (fgets(tmpstr,MAX_LINE,cl_file)!=NULL){
        	
	/*create job file for each station*/
	strncpy(filename,job_filename,MAX_LINE);
	strncat(filename,get_station_name(tmpstr1,tmpstr),MAX_LINE);
	
	job_file=fopen(filename,"a");
    
        if (job_file==NULL){
	    syslog(LOG_INFO, "rsbac-net: Cannot open job file.");
        }
	fwrite(job_line,MAX_LINE,1,job_file);
        fclose(job_file);
    }
}

/*append a new job to job files*/
void append_job_to_station(int ns,char *station,char *session_keyword,char *crypto_keyword)
{
    FILE *job_file=NULL;

    char tmpstr[MAX_LINE];
    char filename[MAX_LINE];
    
    /*create job file for station PATH+station */
    strncpy(filename,job_filename,MAX_LINE);
    strncat(filename,station,MAX_LINE);
	
    job_file=fopen(filename,"a");
    
    if (job_file==NULL){
	syslog(LOG_INFO, "rsbac-net: Cannot open job file.");
    }
		

    if (recv(ns, tmpstr, MAX_LINE, 0)==-1){/*get job from client*/
	syslog(LOG_INFO, "rsbac-net: Cannot recieve message.");
    }
    my_decrypt(session_keyword,tmpstr);/*and decrypt*/
    
#ifdef TEST
    printf(">>+job:%s\n",tmpstr);
#endif
    
    /*encrypt and write job to job-file*/
    my_encrypt(crypto_keyword,tmpstr);
    fwrite(tmpstr,MAX_LINE,1,job_file);
    fclose(job_file);
}

/*append job myself*/
void append_job_mine(char *command_str)
{
    FILE *job_file=NULL;

    char filename[MAX_LINE];
    
    /*create job file for station PATH+current_station*/
    strncpy(filename,job_filename,MAX_LINE);
    strncat(filename,inet_ntoa(clnt_addr.sin_addr),MAX_LINE);
	
    job_file=fopen(filename,"a");
    
    if (job_file==NULL){
	syslog(LOG_INFO, "rsbac-net: Cannot open job file.");
    }
		
    
#ifdef TEST
    printf(">>+job-mine:\n");
#endif
    
    /*encrypt and write job to job-file*/
    fwrite(command_str,MAX_LINE,1,job_file);
    fclose(job_file);
}

/*clear all job-files*/
void clean_all_job()
{
    FILE *cl_file=NULL;
    char filename[MAX_LINE];
    char tmpstr[MAX_LINE];
    char tmpstr1[MAX_LINE];
    
    cl_file=fopen(clientlist_filename,"r");
    
    if (cl_file==NULL)
    {
	syslog(LOG_INFO, "rsbac-net: Cannot open client-list file.");
	exit(1);
    }
    
    /*processing for all clients*/
    while (fgets(tmpstr,MAX_LINE,cl_file)!=NULL){
	/*create job filename for each station*/
	strncpy(filename,job_filename,MAX_LINE);
	strncat(filename,get_station_name(tmpstr1,tmpstr),MAX_LINE);
	
	unlink(filename);	
    }
}

/*clear all jobs to selected station*/
void clean_all_job_to_station(char *station)
{
    char filename[MAX_LINE];
    
    /*create job file-name for station PATH+station */
    strncpy(filename,job_filename,MAX_LINE);
    strncat(filename,station,MAX_LINE);
    
    /*remove station's job-file*/
    unlink(filename);
}

/*open job file for selected station*/
FILE *open_job(char *station)
{
FILE *j_file=NULL;
char filename[MAX_LINE];
    
    strncpy(filename,/*JOB_PATH*/job_filename,MAX_LINE);
    strncat(filename,station,MAX_LINE);
    j_file=fopen(filename,"r");
    
    if (j_file==NULL) {
	syslog(LOG_INFO, "rsbac-net: Cannot open job file.");
	}

    return j_file;
}

/*close and clear station's job file*/
void end_job(FILE *j_file,char *station,int del)
{
    char filename[MAX_LINE];

    strncpy(filename,job_filename,MAX_LINE);
    strncat(filename,station,MAX_LINE);

    fclose(j_file);
    /*remove job-file*/
    
    if (del==1) unlink(filename);
}

/*send to client selected station's jobs*/
void send_jobs_station(int ns,char *session_keyword,char *crypto_keyword,int warranty,char *station)
{
FILE *j_file=NULL;
char data_str[MAX_LINE];
char answer[MAX_LINE];

    j_file=open_job(station);
    
    if (j_file!=NULL){
    
	while (fread(data_str,MAX_LINE,1,j_file)>0){
	    /*crypto*/
	    my_decrypt(crypto_keyword,data_str);
#ifdef TEST
	    printf(">>send:%s\n",data_str);
#endif
	    /*re-crypt using per session keyword*/
	    my_encrypt(session_keyword,data_str);
	    
	    //send data to client 
	    if (send(ns, data_str, MAX_LINE, 0)==-1){
		syslog(LOG_INFO, "rsbac-net: Cannot send message.");
    	    }
	    
	    /*Wait for client answer if Warranty*/
	    if (warranty==1){
	        if (recv(ns, answer, MAX_LINE, 0)==-1){/*receive answer*/
		    syslog(LOG_INFO, "rsbac-net: Cannot send message.");
    		}
		my_decrypt(session_keyword,answer);/*then decrypt*/
		if (!(strncmp(answer,WARRANTY_NO,MAX_LINE))){
		my_decrypt(session_keyword,data_str);/*decrypt job line*/
#ifdef TEST
		    fprintf(stderr,">>warranty:job %s failed for station %s\n",
		    data_str,inet_ntoa(clnt_addr.sin_addr));
#endif
		}
	    }/*warranty*/
	}/*while*/
	
	end_job(j_file,station,1);
    }/*if*/
return;
}


/*send to client it's jobs. Only see*/
void peek_jobs(int ns,char *session_keyword,char *crypto_keyword,char *station)
{
FILE *j_file=NULL;
char data_str[MAX_LINE];

#ifdef TEST
    printf(">>peek:station:%s\n",station);
#endif

    j_file=open_job(station);
    
    if (j_file!=NULL){
    
	while (fread(data_str,MAX_LINE,1,j_file)>0){
	    /*crypto*/
	    my_decrypt(crypto_keyword,data_str);

#ifdef TEST
    printf(">>peek:%s\n",data_str);
#endif
	    /*re-crypt using per session keyword*/
	    my_encrypt(session_keyword,data_str);
	    
	    //send data to client 
	    if (send(ns, data_str, MAX_LINE, 0)==-1){
		syslog(LOG_INFO, "rsbac-net: Cannot send message.");
    	    }
	}/*while*/
	
	end_job(j_file,station,0);
    }/*if*/
return;
}